@@ -28,6 +28,7 @@ import (
 	"os"
 	"os/exec"
 	"path"
+	"path/filepath"
 	"strconv"
 	"strings"
 	"sync"
@@ -36,16 +37,19 @@ import (
 
 	winio "github.com/Microsoft/go-winio"
 	"github.com/Microsoft/hcsshim/pkg/go-runhcs"
+	eventstypes "github.com/containerd/containerd/api/events"
 	containerd_types "github.com/containerd/containerd/api/types"
 	"github.com/containerd/containerd/api/types/task"
 	"github.com/containerd/containerd/errdefs"
-	"github.com/containerd/containerd/events"
 	"github.com/containerd/containerd/log"
 	"github.com/containerd/containerd/mount"
 	"github.com/containerd/containerd/namespaces"
+	"github.com/containerd/containerd/runtime"
+	"github.com/containerd/containerd/runtime/v2/runhcs/options"
 	"github.com/containerd/containerd/runtime/v2/shim"
 	taskAPI "github.com/containerd/containerd/runtime/v2/task"
 	"github.com/containerd/go-runc"
+	"github.com/containerd/typeurl"
 	ptypes "github.com/gogo/protobuf/types"
 	oci "github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/pkg/errors"
@@ -63,18 +67,6 @@ var (
 	empty = &ptypes.Empty{}
 )
 
-func newRunhcs(debugLog string) *runhcs.Runhcs {
-	rhs := &runhcs.Runhcs{
-		Debug:     debugLog != "",
-		LogFormat: runhcs.JSON,
-		Owner:     "containerd-runhcs-shim-v1",
-	}
-	if rhs.Debug {
-		rhs.Log = debugLog
-	}
-	return rhs
-}
-
 // forwardRunhcsLogs copies logs from c and writes them to the ctx logger
 // upstream.
 func forwardRunhcsLogs(ctx context.Context, c net.Conn, fields logrus.Fields) {
@@ -109,7 +101,7 @@ func forwardRunhcsLogs(ctx context.Context, c net.Conn, fields logrus.Fields) {
 
 		// TODO: JTERRY75 maybe we need to make this configurable so we know
 		// that runhcs is using the same one we are deserializing.
-		ti, err := time.Parse(logrus.DefaultTimestampFormat, e.Data[logrus.FieldKeyTime].(string))
+		ti, err := time.Parse(time.RFC3339, e.Data[logrus.FieldKeyTime].(string))
 		if err != nil {
 			log.G(ctx).WithFields(fields).WithError(err).Debug("invalid time stamp format")
 			ti = time.Time{}
@@ -136,12 +128,13 @@ func forwardRunhcsLogs(ctx context.Context, c net.Conn, fields logrus.Fields) {
 }
 
 // New returns a new runhcs shim service that can be used via GRPC
-func New(ctx context.Context, id string, publisher events.Publisher) (shim.Shim, error) {
+func New(ctx context.Context, id string, publisher shim.Publisher, shutdown func()) (shim.Shim, error) {
 	return &service{
 		context:   ctx,
 		id:        id,
 		processes: make(map[string]*process),
 		publisher: publisher,
+		shutdown:  shutdown,
 	}, nil
 }
 
@@ -154,14 +147,30 @@ type service struct {
 
 	context context.Context
 
-	// debugLog if not "" indicates the log pipe path for runhcs.exe to write its logs to.
-	debugLog      string
+	// debugLog if not "" indicates the log file or pipe path for runhcs.exe to
+	// write its logs to.
+	debugLog string
+	// if `shimOpts.DebugType == options.Opitons_NPIPE` will hold the listener
+	// for the runhcs.exe to connect to for sending logs.
 	debugListener net.Listener
 
+	shimOpts options.Options
+
 	id        string
 	processes map[string]*process
 
-	publisher events.Publisher
+	publisher shim.Publisher
+	shutdown  func()
+}
+
+func (s *service) newRunhcs() *runhcs.Runhcs {
+	return &runhcs.Runhcs{
+		Debug:     s.shimOpts.Debug,
+		Log:       s.debugLog,
+		LogFormat: runhcs.JSON,
+		Owner:     "containerd-runhcs-shim-v1",
+		Root:      s.shimOpts.RegistryRoot,
+	}
 }
 
 // getProcess attempts to get a process by id.
@@ -196,7 +205,7 @@ func (s *service) Cleanup(ctx context.Context) (*taskAPI.DeleteResponse, error)
 		return nil, err
 	}
 	// Forcibly shut down any container in this bundle
-	rhcs := newRunhcs("")
+	rhcs := s.newRunhcs()
 	dopts := &runhcs.DeleteOpts{
 		Force: true,
 	}
@@ -290,7 +299,7 @@ func (s *service) State(ctx context.Context, r *taskAPI.StateRequest) (*taskAPI.
 
 	// This is a container
 	if p.cid == p.id {
-		rhcs := newRunhcs(s.debugLog)
+		rhcs := s.newRunhcs()
 		cs, err := rhcs.State(ctx, p.id)
 		if err != nil {
 			return nil, err
@@ -319,7 +328,7 @@ func (s *service) State(ctx context.Context, r *taskAPI.StateRequest) (*taskAPI.
 	return &taskAPI.StateResponse{
 		ID:         p.id,
 		Bundle:     p.bundle,
-		Pid:        p.pid,
+		Pid:        pe.pid,
 		Status:     status,
 		Stdin:      p.stdin,
 		Stdout:     p.stdout,
@@ -331,7 +340,25 @@ func (s *service) State(ctx context.Context, r *taskAPI.StateRequest) (*taskAPI.
 }
 
 func writeMountsToConfig(bundle string, mounts []*containerd_types.Mount) error {
-	if len(mounts) != 1 {
+	cf, err := os.OpenFile(path.Join(bundle, "config.json"), os.O_RDWR, 0)
+	if err != nil {
+		return errors.Wrap(err, "bundle does not contain config.json")
+	}
+	defer cf.Close()
+	var spec oci.Spec
+	if err := json.NewDecoder(cf).Decode(&spec); err != nil {
+		return errors.Wrap(err, "bundle config.json is not valid oci spec")
+	}
+
+	if len(mounts) == 0 {
+		// If no mounts are passed via the snapshotter its the callers full
+		// responsibility to manage the storage. Just move on without affecting
+		// the config.json at all.
+		if spec.Windows == nil || len(spec.Windows.LayerFolders) < 2 {
+			return errors.New("no Windows.LayerFolders found in oci spec")
+		}
+		return nil
+	} else if len(mounts) != 1 {
 		return errors.New("Rootfs does not contain exactly 1 mount for the root file system")
 	}
 
@@ -340,11 +367,11 @@ func writeMountsToConfig(bundle string, mounts []*containerd_types.Mount) error
 		return errors.Errorf("unsupported mount type '%s'", m.Type)
 	}
 
-	// parentLayerPaths are passed in layerN, layerN-1, ..., layer 0
+	// parentLayerPaths are passed in:
+	//   layerN, layerN-1, ..., layer0
 	//
 	// The OCI spec expects:
-	//   windows-layer order is layerN, layerN-1, ..., layer0, scratch
-	//   lcow-layer    order is layer0,   layer1, ..., layerN, scratch
+	//   layerN, layerN-1, ..., layer0, scratch
 	var parentLayerPaths []string
 	for _, option := range mounts[0].Options {
 		if strings.HasPrefix(option, mount.ParentLayerPathsFlag) {
@@ -356,34 +383,18 @@ func writeMountsToConfig(bundle string, mounts []*containerd_types.Mount) error
 	}
 
 	if m.Type == "lcow-layer" {
-		// Reverse the lcow-layer parents
-		for i := len(parentLayerPaths)/2 - 1; i >= 0; i-- {
-			opp := len(parentLayerPaths) - 1 - i
-			parentLayerPaths[i], parentLayerPaths[opp] = parentLayerPaths[opp], parentLayerPaths[i]
+		// If we are creating LCOW make sure that spec.Windows is filled out before
+		// appending layer folders.
+		if spec.Windows == nil {
+			spec.Windows = &oci.Windows{}
 		}
-	}
-
-	cf, err := os.OpenFile(path.Join(bundle, "config.json"), os.O_RDWR, 0)
-	if err != nil {
-		return errors.Wrap(err, "bundle does not contain config.json")
-	}
-	defer cf.Close()
-	var spec oci.Spec
-	if err := json.NewDecoder(cf).Decode(&spec); err != nil {
-		return errors.Wrap(err, "bundle config.json is not valid oci spec")
-	}
-	if err := cf.Truncate(0); err != nil {
-		return errors.Wrap(err, "failed to truncate config.json")
-	}
-	if _, err := cf.Seek(0, 0); err != nil {
-		return errors.Wrap(err, "failed to seek to 0 in config.json")
-	}
-
-	// If we are creating LCOW make sure that spec.Windows is filled out before
-	// appending layer folders.
-	if m.Type == "lcow-layer" && spec.Windows == nil {
-		spec.Windows = &oci.Windows{
-			HyperV: &oci.WindowsHyperV{},
+		if spec.Windows.HyperV == nil {
+			spec.Windows.HyperV = &oci.WindowsHyperV{}
+		}
+	} else if spec.Windows.HyperV == nil {
+		// This is a Windows Argon make sure that we have a Root filled in.
+		if spec.Root == nil {
+			spec.Root = &oci.Root{}
 		}
 	}
 
@@ -392,6 +403,13 @@ func writeMountsToConfig(bundle string, mounts []*containerd_types.Mount) error
 	// Append the scratch
 	spec.Windows.LayerFolders = append(spec.Windows.LayerFolders, m.Source)
 
+	if err := cf.Truncate(0); err != nil {
+		return errors.Wrap(err, "failed to truncate config.json")
+	}
+	if _, err := cf.Seek(0, 0); err != nil {
+		return errors.Wrap(err, "failed to seek to 0 in config.json")
+	}
+
 	if err := json.NewEncoder(cf).Encode(spec); err != nil {
 		return errors.Wrap(err, "failed to write Mounts into config.json")
 	}
@@ -406,6 +424,19 @@ func (s *service) Create(ctx context.Context, r *taskAPI.CreateTaskRequest) (*ta
 	s.mu.Lock()
 	defer s.mu.Unlock()
 
+	_, err := namespaces.NamespaceRequired(ctx)
+	if err != nil {
+		return nil, errors.Wrap(err, "create namespace")
+	}
+
+	if r.Options != nil {
+		v, err := typeurl.UnmarshalAny(r.Options)
+		if err != nil {
+			return nil, err
+		}
+		s.shimOpts = *v.(*options.Options)
+	}
+
 	if p := s.processes[r.ID]; p != nil {
 		return nil, errdefs.ToGRPCf(errdefs.ErrAlreadyExists, "process %s already exists", r.ID)
 	}
@@ -445,38 +476,41 @@ func (s *service) Create(ctx context.Context, r *taskAPI.CreateTaskRequest) (*ta
 			pr.close()
 		}
 	}()
-	// TODO: Parse the real RunHcs Opts r.Options
-	opts, ok := ctx.Value(shim.OptsKey{}).(shim.Opts)
-	if ok && opts.Debug {
-		if s.debugLog == "" {
-			logPath := safePipePrefix + fmt.Sprintf("runhcs-log-%s", r.ID)
-			l, err := winio.ListenPipe(logPath, nil)
-			if err != nil {
-				return nil, err
-			}
-			s.debugLog = logPath
-			s.debugListener = l
 
-			// Accept connections and forward all logs for each runhcs.exe
-			// invocation
-			go func() {
-				for {
-					c, err := s.debugListener.Accept()
-					if err != nil {
-						if err == errorConnectionAborted {
-							break
-						}
-						log.G(ctx).WithError(err).Debug("log accept failure")
-						// Logrus error locally?
-						continue
-					}
-					fields := map[string]interface{}{
-						"log-source": "runhcs",
-						"task-id":    r.ID,
-					}
-					go forwardRunhcsLogs(ctx, c, fields)
+	if s.shimOpts.Debug {
+		if s.debugLog == "" {
+			if s.shimOpts.DebugType == options.Options_FILE {
+				s.debugLog = filepath.Join(r.Bundle, fmt.Sprintf("runhcs-%s.log", r.ID))
+			} else {
+				logPath := safePipePrefix + fmt.Sprintf("runhcs-log-%s", r.ID)
+				l, err := winio.ListenPipe(logPath, nil)
+				if err != nil {
+					return nil, err
 				}
-			}()
+				s.debugLog = logPath
+				s.debugListener = l
+
+				// Accept connections and forward all logs for each runhcs.exe
+				// invocation
+				go func() {
+					for {
+						c, err := s.debugListener.Accept()
+						if err != nil {
+							if err == winio.ErrPipeListenerClosed {
+								break
+							}
+							log.G(ctx).WithError(err).Debug("log accept failure")
+							// Logrus error locally?
+							continue
+						}
+						fields := map[string]interface{}{
+							"log-source": "runhcs",
+							"task-id":    r.ID,
+						}
+						go forwardRunhcsLogs(ctx, c, fields)
+					}
+				}()
+			}
 		}
 	}
 
@@ -485,42 +519,47 @@ func (s *service) Create(ctx context.Context, r *taskAPI.CreateTaskRequest) (*ta
 		IO:      io,
 		PidFile: pidfilePath,
 	}
-	rhcs := newRunhcs(s.debugLog)
+	rhcs := s.newRunhcs()
 	if rhcs.Debug {
-		doForwardLogs := func(source, logPipeFmt string, opt *string) error {
-			pipeName := fmt.Sprintf(logPipeFmt, r.ID)
-			*opt = safePipePrefix + pipeName
-			l, err := winio.ListenPipe(*opt, nil)
-			if err != nil {
-				return err
-			}
-			go func() {
-				defer l.Close()
-				c, err := l.Accept()
+		if s.shimOpts.DebugType == options.Options_FILE {
+			copts.ShimLog = filepath.Join(r.Bundle, fmt.Sprintf("runhcs-shim-%s.log", r.ID))
+			copts.VMLog = filepath.Join(r.Bundle, fmt.Sprintf("runhcs-vmshim-%s.log", r.ID))
+		} else {
+			doForwardLogs := func(source, logPipeFmt string, opt *string) error {
+				pipeName := fmt.Sprintf(logPipeFmt, r.ID)
+				*opt = safePipePrefix + pipeName
+				l, err := winio.ListenPipe(*opt, nil)
 				if err != nil {
-					log.G(ctx).
-						WithField("task-id", r.ID).
-						WithError(err).
-						Errorf("failed to accept %s", pipeName)
-				} else {
-					fields := map[string]interface{}{
-						"log-source": source,
-						"task-id":    r.ID,
-					}
-					go forwardRunhcsLogs(ctx, c, fields)
+					return err
 				}
-			}()
-			return nil
-		}
+				go func() {
+					defer l.Close()
+					c, err := l.Accept()
+					if err != nil {
+						log.G(ctx).
+							WithField("task-id", r.ID).
+							WithError(err).
+							Errorf("failed to accept %s", pipeName)
+					} else {
+						fields := map[string]interface{}{
+							"log-source": source,
+							"task-id":    r.ID,
+						}
+						go forwardRunhcsLogs(ctx, c, fields)
+					}
+				}()
+				return nil
+			}
 
-		err = doForwardLogs("runhcs-shim", "runhcs-shim-log-%s", &copts.ShimLog)
-		if err != nil {
-			return nil, err
-		}
+			err = doForwardLogs("runhcs-shim", "runhcs-shim-log-%s", &copts.ShimLog)
+			if err != nil {
+				return nil, err
+			}
 
-		err = doForwardLogs("runhcs--vm-shim", "runhcs-vm-shim-log-%s", &copts.VMLog)
-		if err != nil {
-			return nil, err
+			err = doForwardLogs("runhcs--vm-shim", "runhcs-vm-shim-log-%s", &copts.VMLog)
+			if err != nil {
+				return nil, err
+			}
 		}
 	}
 	err = rhcs.Create(ctx, r.ID, r.Bundle, copts)
@@ -551,6 +590,22 @@ func (s *service) Create(ctx context.Context, r *taskAPI.CreateTaskRequest) (*ta
 	}
 	s.processes[r.ID] = process
 
+	s.publisher.Publish(ctx,
+		runtime.TaskCreateEventTopic,
+		&eventstypes.TaskCreate{
+			ContainerID: process.id,
+			Bundle:      process.bundle,
+			Rootfs:      r.Rootfs,
+			IO: &eventstypes.TaskIO{
+				Stdin:    r.Stdin,
+				Stdout:   r.Stdout,
+				Stderr:   r.Stderr,
+				Terminal: r.Terminal,
+			},
+			Checkpoint: "",
+			Pid:        uint32(pid),
+		})
+
 	return &taskAPI.CreateTaskResponse{
 		Pid: uint32(pid),
 	}, nil
@@ -571,7 +626,7 @@ func (s *service) Start(ctx context.Context, r *taskAPI.StartRequest) (*taskAPI.
 		return nil, errors.New("cannot start already started container or process")
 	}
 
-	rhcs := newRunhcs(s.debugLog)
+	rhcs := s.newRunhcs()
 
 	// This is a start/exec
 	if r.ExecID != "" {
@@ -585,39 +640,54 @@ func (s *service) Start(ctx context.Context, r *taskAPI.StartRequest) (*taskAPI.
 		}
 
 		if rhcs.Debug {
-			doForwardLogs := func(source, pipeName string, opt *string) error {
-				*opt = safePipePrefix + pipeName
-				l, err := winio.ListenPipe(*opt, nil)
-				if err != nil {
-					return err
-				}
-				go func() {
-					defer l.Close()
-					c, err := l.Accept()
+			if s.shimOpts.DebugType == options.Options_FILE {
+				eopts.ShimLog = filepath.Join(p.bundle, fmt.Sprintf("runhcs-shim-%s.log", execFmt))
+			} else {
+				doForwardLogs := func(source, pipeName string, opt *string) error {
+					*opt = safePipePrefix + pipeName
+					l, err := winio.ListenPipe(*opt, nil)
 					if err != nil {
-						log.G(ctx).
-							WithField("task-id", r.ID).
-							WithField("exec-id", r.ExecID).
-							WithError(err).
-							Errorf("failed to accept %s", pipeName)
-					} else {
-						fields := map[string]interface{}{
-							"log-source": source,
-							"task-id":    r.ID,
-							"exec-id":    r.ExecID,
-						}
-						go forwardRunhcsLogs(ctx, c, fields)
+						return err
 					}
-				}()
-				return nil
-			}
+					go func() {
+						defer l.Close()
+						c, err := l.Accept()
+						if err != nil {
+							log.G(ctx).
+								WithField("task-id", r.ID).
+								WithField("exec-id", r.ExecID).
+								WithError(err).
+								Errorf("failed to accept %s", pipeName)
+						} else {
+							fields := map[string]interface{}{
+								"log-source": source,
+								"task-id":    r.ID,
+								"exec-id":    r.ExecID,
+							}
+							go forwardRunhcsLogs(ctx, c, fields)
+						}
+					}()
+					return nil
+				}
 
-			err = doForwardLogs("runhcs-shim-exec", "runhcs-shim-log-"+execFmt, &eopts.ShimLog)
-			if err != nil {
-				return nil, err
+				err = doForwardLogs("runhcs-shim-exec", "runhcs-shim-log-"+execFmt, &eopts.ShimLog)
+				if err != nil {
+					return nil, err
+				}
 			}
 		}
 
+		// For the exec case increment the startedWg to make sure we publish the
+		// started event previous to the exit published in `waitForProcess`.
+		p.startedWg.Add(1)
+		defer func() {
+			if err != nil {
+				// If an exec failure takes place decrement the startedWg so we
+				// dont get off on counts.
+				p.startedWg.Done()
+			}
+		}()
+
 		// ID here is the containerID to exec the process in.
 		err = rhcs.Exec(ctx, r.ID, procConfig, eopts)
 		if err != nil {
@@ -636,7 +706,6 @@ func (s *service) Start(ctx context.Context, r *taskAPI.StartRequest) (*taskAPI.
 		if err != nil {
 			return nil, errors.Wrap(err, "failed to find exec process pid")
 		}
-		p.pid = uint32(pid)
 		go waitForProcess(ctx, p, proc, s)
 	} else {
 		if err := rhcs.Start(ctx, p.id); err != nil {
@@ -666,8 +735,29 @@ func (s *service) Start(ctx context.Context, r *taskAPI.StartRequest) (*taskAPI.
 			time.Sleep(1 * time.Second)
 		}
 	}
+
+	pid := p.stat().pid
+	if r.ExecID != "" {
+		s.publisher.Publish(ctx,
+			runtime.TaskExecStartedEventTopic,
+			&eventstypes.TaskExecStarted{
+				ContainerID: p.cid,
+				ExecID:      p.id,
+				Pid:         pid,
+			})
+	} else {
+		s.publisher.Publish(ctx,
+			runtime.TaskStartEventTopic,
+			&eventstypes.TaskStart{
+				ContainerID: p.id,
+				Pid:         pid,
+			})
+	}
+
+	p.startedWg.Done()
+
 	return &taskAPI.StartResponse{
-		Pid: p.pid,
+		Pid: pid,
 	}, nil
 }
 
@@ -683,7 +773,7 @@ func (s *service) Delete(ctx context.Context, r *taskAPI.DeleteRequest) (*taskAP
 
 	// This is a container
 	if p.cid == p.id {
-		rhs := newRunhcs(s.debugLog)
+		rhs := s.newRunhcs()
 		dopts := &runhcs.DeleteOpts{
 			Force: true,
 		}
@@ -704,10 +794,18 @@ func (s *service) Delete(ctx context.Context, r *taskAPI.DeleteRequest) (*taskAP
 	s.mu.Lock()
 	delete(s.processes, p.id)
 	s.mu.Unlock()
+
+	s.publisher.Publish(ctx, runtime.TaskDeleteEventTopic, &eventstypes.TaskDelete{
+		ContainerID: p.id,
+		Pid:         exit.pid,
+		ExitStatus:  exit.exitStatus,
+		ExitedAt:    exit.exitedAt,
+	})
+
 	return &taskAPI.DeleteResponse{
 		ExitedAt:   exit.exitedAt,
 		ExitStatus: exit.exitStatus,
-		Pid:        p.pid,
+		Pid:        exit.pid,
 	}, nil
 }
 
@@ -729,11 +827,15 @@ func (s *service) Pause(ctx context.Context, r *taskAPI.PauseRequest) (*ptypes.E
 		return nil, err
 	}
 
-	rhcs := newRunhcs(s.debugLog)
+	rhcs := s.newRunhcs()
 	if err = rhcs.Pause(ctx, p.id); err != nil {
 		return nil, err
 	}
 
+	s.publisher.Publish(ctx, runtime.TaskPausedEventTopic, &eventstypes.TaskPaused{
+		ContainerID: r.ID,
+	})
+
 	return empty, nil
 }
 
@@ -748,11 +850,15 @@ func (s *service) Resume(ctx context.Context, r *taskAPI.ResumeRequest) (*ptypes
 		return nil, err
 	}
 
-	rhcs := newRunhcs(s.debugLog)
+	rhcs := s.newRunhcs()
 	if err = rhcs.Resume(ctx, p.id); err != nil {
 		return nil, err
 	}
 
+	s.publisher.Publish(ctx, runtime.TaskResumedEventTopic, &eventstypes.TaskResumed{
+		ContainerID: r.ID,
+	})
+
 	return empty, nil
 }
 
@@ -773,9 +879,8 @@ func (s *service) Kill(ctx context.Context, r *taskAPI.KillRequest) (*ptypes.Emp
 		return nil, err
 	}
 
-	// TODO: JTERRY75 runhcs needs r.Signal in string form
 	// TODO: JTERRY75 runhcs support for r.All?
-	rhcs := newRunhcs(s.debugLog)
+	rhcs := s.newRunhcs()
 	if err = rhcs.Kill(ctx, p.id, strconv.FormatUint(uint64(r.Signal), 10)); err != nil {
 		if !strings.Contains(err.Error(), "container is stopped") {
 			return nil, err
@@ -865,6 +970,13 @@ func (s *service) Exec(ctx context.Context, r *taskAPI.ExecProcessRequest) (*pty
 	}
 	s.processes[r.ExecID] = process
 
+	s.publisher.Publish(ctx,
+		runtime.TaskExecAddedEventTopic,
+		&eventstypes.TaskExecAdded{
+			ContainerID: process.cid,
+			ExecID:      process.id,
+		})
+
 	return empty, nil
 }
 
@@ -878,11 +990,11 @@ func (s *service) ResizePty(ctx context.Context, r *taskAPI.ResizePtyRequest) (*
 		return nil, err
 	}
 
-	pid := int(p.pid)
+	pid := int(p.stat().pid)
 	opts := runhcs.ResizeTTYOpts{
 		Pid: &pid,
 	}
-	rhcs := newRunhcs(s.debugLog)
+	rhcs := s.newRunhcs()
 	if err = rhcs.ResizeTTY(ctx, p.cid, uint16(r.Width), uint16(r.Height), &opts); err != nil {
 		return nil, err
 	}
@@ -937,9 +1049,15 @@ func (s *service) Stats(ctx context.Context, r *taskAPI.StatsRequest) (*taskAPI.
 func (s *service) Connect(ctx context.Context, r *taskAPI.ConnectRequest) (*taskAPI.ConnectResponse, error) {
 	log.G(ctx).Debugf("Connect: %s", r.ID)
 
+	var taskpid uint32
+	p, _ := s.getProcess(r.ID, "")
+	if p != nil {
+		taskpid = p.stat().pid
+	}
+
 	return &taskAPI.ConnectResponse{
 		ShimPid: uint32(os.Getpid()),
-		TaskPid: s.processes[s.id].pid,
+		TaskPid: taskpid,
 		Version: runhcsShimVersion,
 	}, nil
 }
@@ -951,7 +1069,8 @@ func (s *service) Shutdown(ctx context.Context, r *taskAPI.ShutdownRequest) (*pt
 	if s.debugListener != nil {
 		s.debugListener.Close()
 	}
+	s.publisher.Close()
+	s.shutdown()
 
-	os.Exit(0)
 	return empty, nil
 }
